﻿using System;
using System.IO;
using System.Linq;
using Xunit;

namespace Kiota.Builder.Writers.Shell.Tests;

public class ShellCodeMethodWriterTests : IDisposable
{
    private const string DefaultPath = "./";
    private const string DefaultName = "name";
    private readonly StringWriter tw;
    private readonly LanguageWriter writer;
    private readonly CodeMethod method;
    private readonly CodeClass parentClass;
    private readonly CodeNamespace root;
    private const string MethodName = "methodName";
    private const string ReturnTypeName = "Somecustomtype";
    private const string MethodDescription = "some description";
    private const string ParamDescription = "some parameter description";
    private const string ParamName = "paramName";

    public ShellCodeMethodWriterTests()
    {
        writer = LanguageWriter.GetLanguageWriter(GenerationLanguage.Shell, DefaultPath, DefaultName);
        tw = new StringWriter();
        writer.SetTextWriter(tw);
        root = CodeNamespace.InitRootNamespace();
        root.Name = "Test";
        parentClass = new CodeClass
        {
            Name = "parentClass"
        };
        root.AddClass(parentClass);
        method = new CodeMethod
        {
            Name = MethodName
        };
        method.ReturnType = new CodeType
        {
            Name = ReturnTypeName
        };
        parentClass.AddMethod(method);
    }

    public void Dispose()
    {
        tw?.Dispose();
        GC.SuppressFinalize(this);
    }

    private void AddRequestProperties() {
        parentClass.AddProperty(new CodeProperty {
            Name = "RequestAdapter",
            Kind = CodePropertyKind.RequestAdapter,
        });
        parentClass.AddProperty(new CodeProperty {
            Name = "pathParameters",
            Kind = CodePropertyKind.PathParameters,
        });
        parentClass.AddProperty(new CodeProperty {
            Name = "urlTemplate",
            Kind = CodePropertyKind.UrlTemplate,
        });
    }

    private static void AddRequestBodyParameters(CodeMethod method) {
        var stringType = new CodeType {
            Name = "string",
        };
        method.AddParameter(new CodeParameter {
            Name = "h",
            Kind = CodeParameterKind.Headers,
            Type = stringType,
        });
        method.AddParameter(new CodeParameter{
            Name = "q",
            Kind = CodeParameterKind.QueryParameter,
            Type = stringType,
        });
        method.AddParameter(new CodeParameter{
            Name = "b",
            Kind = CodeParameterKind.RequestBody,
            Type = stringType,
        });
        method.AddParameter(new CodeParameter{
            Name = "r",
            Kind = CodeParameterKind.ResponseHandler,
            Type = stringType,
        });
        method.AddParameter(new CodeParameter {
            Name = "o",
            Kind = CodeParameterKind.Options,
            Type = stringType,
        });
        method.AddParameter(new CodeParameter
        {
            Name = "c",
            Kind = CodeParameterKind.Cancellation,
            Type = stringType,
        });
    }

    private static void AddPathQueryAndHeaderParameters(CodeMethod method) {
        var stringType = new CodeType {
            Name = "string",
        };
        method.AddPathQueryOrHeaderParameter(new CodeParameter{
            Name = "q",
            Kind = CodeParameterKind.QueryParameter,
            Type = stringType,
            DefaultValue = "test",
            Description = "The q option",
            Optional = true
        });
        method.AddPathQueryOrHeaderParameter(new CodeParameter {
            Name = "test-path",
            Kind = CodeParameterKind.Path,
            Type = stringType
        });
        method.AddPathQueryOrHeaderParameter(new CodeParameter {
            Name = "Test-Header",
            Kind = CodeParameterKind.Headers,
            Type = stringType,
            Description = "The test header",
        });
    }

    [Fact]
    public void WritesRootCommand()
    {
        method.Kind = CodeMethodKind.CommandBuilder;
        method.OriginalMethod = new CodeMethod
        {
            Kind = CodeMethodKind.ClientConstructor
        };

        writer.Write(method);

        var result = tw.ToString();

        Assert.Contains("var command = new RootCommand();", result);

        Assert.Contains("return command;", result);
    }

    [Fact]
    public void WritesRootCommandWithCommandBuilderMethods()
    {
        method.Kind = CodeMethodKind.CommandBuilder;
        method.OriginalMethod = new CodeMethod
        {
            Kind = CodeMethodKind.ClientConstructor
        };
        parentClass.AddMethod(new CodeMethod {
            Name = "BuildUserCommand",
            Kind = CodeMethodKind.CommandBuilder
        });

        writer.Write(method);

        var result = tw.ToString();

        Assert.Contains("var command = new RootCommand();", result);
        Assert.Contains("command.AddCommand(BuildUserCommand());", result);
        Assert.Contains("return command;", result);
    }

    [Fact]
    public void WritesIndexerCommands() {
        method.Kind = CodeMethodKind.CommandBuilder;
        var type = new CodeClass { Name = "TestClass", Kind = CodeClassKind.RequestBuilder };
        type.AddMethod(new CodeMethod { Kind = CodeMethodKind.CommandBuilder, Name = "BuildTestMethod1", ReturnType = new CodeType() });
        type.AddMethod(new CodeMethod { Kind = CodeMethodKind.CommandBuilder, Name = "BuildTestMethod2", ReturnType = new CodeType {CollectionKind = CodeTypeBase.CodeTypeCollectionKind.Array} });
        type.Parent = CodeNamespace.InitRootNamespace();
        type.Parent.Name = "Test.Name.Sub";
        method.OriginalIndexer = new CodeIndexer {
            ReturnType = new CodeType {
                Name = "TestRequestBuilder",
                TypeDefinition = type
            }
        };

        AddRequestProperties();

        writer.Write(method);
        var result = tw.ToString();

        Assert.Contains("var builder = new TestRequestBuilder", result);
        Assert.Contains("var command = new Command(\"item\");", result);
        Assert.Contains("command.AddCommand(builder.BuildTestMethod1());", result);
        Assert.Contains("foreach (var cmd in builder.BuildTestMethod2()) {", result);
        Assert.Contains("command.AddCommand(cmd);", result);
        Assert.Contains("return command;", result);
    }

    [Fact]
    public void WritesContainerCommands()
    {
        method.Kind = CodeMethodKind.CommandBuilder;
        method.SimpleName = "User";
        // Types: A.B.C.T2
        //        A.B.C.D.T1
        var ns1 = root.AddNamespace("Test.Name");
        var ns2 = ns1.AddNamespace("Test.Name.Sub1");
        var ns3 = ns2.AddNamespace("Test.Name.Sub1.Sub2");
        var t2 = parentClass;
        ns2.AddClass(t2);
        var t1Sub = new CodeClass { Name = "TestClass1", Kind = CodeClassKind.RequestBuilder };
        ns3.AddClass(t1Sub);
        t1Sub.AddMethod(new CodeMethod { Kind = CodeMethodKind.CommandBuilder, Name = "BuildTestMethod1", ReturnType = new CodeType() });
        t1Sub.AddMethod(new CodeMethod { Kind = CodeMethodKind.CommandBuilder, Name = "BuildTestMethod2", ReturnType = new CodeType { CollectionKind = CodeTypeBase.CodeTypeCollectionKind.Array } });
        method.AccessedProperty = new CodeProperty
        {
            Type = new CodeType
            {
                Name = "TestRequestBuilder",
                TypeDefinition = t1Sub
            }
        };

        AddRequestProperties();

        writer.Write(method);
        var result = tw.ToString();

        Assert.Contains("var command = new Command(\"user\");", result);
        Assert.Contains("var builder = new TestRequestBuilder", result);
        Assert.Contains("command.AddCommand(builder.BuildTestMethod1());", result);
        Assert.Contains("foreach (var cmd in builder.BuildTestMethod2()) {", result);
        Assert.Contains("command.AddCommand(cmd);", result);
        Assert.Contains("return command;", result);
    }

    [Fact]
    public void WritesContainerCommandWithConflictingTypes()
    {
        method.Kind = CodeMethodKind.CommandBuilder;
        method.SimpleName = "User";
        // Types: A.B.T1
        //        A.B.C.T2
        //        A.B.C.D.T1
        var ns1 = root.AddNamespace("Test.Name");
        var ns2 = ns1.AddNamespace("Test.Name.Sub1");
        var ns3 = ns2.AddNamespace("Test.Name.Sub1.Sub2");
        var t1 = new CodeClass { Name = "TestClass1", Kind = CodeClassKind.RequestBuilder };
        ns1.AddClass(t1);
        var t2 = parentClass;
        ns2.AddClass(t2);
        var t1Sub = new CodeClass { Name = "TestClass1", Kind = CodeClassKind.RequestBuilder };
        ns3.AddClass(t1Sub);
        t1Sub.AddMethod(new CodeMethod { Kind = CodeMethodKind.CommandBuilder, Name = "BuildTestMethod1", ReturnType = new CodeType() });
        t1Sub.AddMethod(new CodeMethod { Kind = CodeMethodKind.CommandBuilder, Name = "BuildTestMethod2", ReturnType = new CodeType { CollectionKind = CodeTypeBase.CodeTypeCollectionKind.Array } });
        method.AccessedProperty = new CodeProperty
        {
            Type = new CodeType
            {
                Name = "TestRequestBuilder",
                TypeDefinition = t1Sub
            }
        };

        AddRequestProperties();

        writer.Write(method);
        var result = tw.ToString();

        Assert.Contains("var command = new Command(\"user\");", result);
        Assert.Contains("var builder = new Test.Name.Sub1.Sub2.TestRequestBuilder", result);
        Assert.Contains("command.AddCommand(builder.BuildTestMethod1());", result);
        Assert.Contains("foreach (var cmd in builder.BuildTestMethod2()) {", result);
        Assert.Contains("command.AddCommand(cmd);", result);
        Assert.Contains("return command;", result);
    }

    [Fact]
    public void WritesExecutableCommandForGetRequestPrimitive()
    {

        method.Kind = CodeMethodKind.CommandBuilder;
        method.Description = "Test description";
        method.SimpleName = "User";
        method.HttpMethod = HttpMethod.Get;
        var stringType = new CodeType
        {
            Name = "string",
        };
        var generatorMethod = new CodeMethod
        {
            Kind = CodeMethodKind.RequestGenerator,
            Name = "CreateGetRequestInformation",
            HttpMethod = method.HttpMethod
        };
        method.OriginalMethod = new CodeMethod
        {
            Kind = CodeMethodKind.RequestExecutor,
            HttpMethod = method.HttpMethod,
            ReturnType = stringType,
            Parent = method.Parent
        };
        var codeClass = method.Parent as CodeClass;
        codeClass.AddMethod(generatorMethod);

        AddRequestProperties();
        AddRequestBodyParameters(method.OriginalMethod);
        AddPathQueryAndHeaderParameters(generatorMethod);

        writer.Write(method);
        var result = tw.ToString();

        Assert.Contains("var command = new Command(\"user\");", result);
        Assert.Contains("command.Description = \"Test description\";", result);
        Assert.Contains("var qOption = new Option<string>(\"-q\", getDefaultValue: ()=> \"test\", description: \"The q option\")", result);
        Assert.Contains("qOption.IsRequired = false;", result);
        Assert.Contains("command.AddOption(qOption);", result);
        Assert.Contains("var testHeaderOption = new Option<string>(\"--test-header\", description: \"The test header\")", result);
        Assert.Contains("testHeaderOption.IsRequired = true;", result);
        Assert.Contains("command.AddOption(testHeaderOption);", result);
        Assert.Contains("command.SetHandler(async (object[] parameters) => {", result);
        Assert.Contains("var q = (string) parameters[0];", result);
        Assert.Contains("var testHeader = (string) parameters[2];", result);
        Assert.Contains("var requestInfo = CreateGetRequestInformation", result);
        Assert.Contains("requestInfo.PathParameters.Add(\"test%2Dpath\", testPath);", result);
        Assert.Contains("requestInfo.Headers[\"Test-Header\"] = testHeader;", result);
        Assert.Contains("var response = await RequestAdapter.SendPrimitiveAsync<Stream>(requestInfo, errorMapping: default, cancellationToken: cancellationToken);", result);
        Assert.Contains("var outputFormatterFactory = (IOutputFormatterFactory) parameters[4];", result);
        Assert.Contains("var formatter = outputFormatterFactory.GetFormatter(FormatterType.TEXT);", result);
        Assert.Contains("await formatter.WriteOutputAsync(response, null, cancellationToken);", result);
        Assert.Contains("}, new CollectionBinding(qOption, testPathOption, testHeaderOption", result);
        Assert.Contains("return command;", result);
    }

    [Fact]
    public void WritesExecutableCommandForGetRequestModel()
    {

        method.Kind = CodeMethodKind.CommandBuilder;
        method.Description = "Test description";
        method.SimpleName = "User";
        method.HttpMethod = HttpMethod.Get;
        var userClass = root.AddClass(new CodeClass
        {
            Name = "User",
            Kind = CodeClassKind.Model
        }).First();
        var stringType = new CodeType
        {
            Name = "user",
            TypeDefinition = userClass,
        };
        var generatorMethod = new CodeMethod
        {
            Kind = CodeMethodKind.RequestGenerator,
            Name = "CreateGetRequestInformation",
            HttpMethod = method.HttpMethod
        };
        method.OriginalMethod = new CodeMethod
        {
            Kind = CodeMethodKind.RequestExecutor,
            HttpMethod = method.HttpMethod,
            ReturnType = stringType,
            Parent = method.Parent
        };
        var codeClass = method.Parent as CodeClass;
        codeClass.AddMethod(generatorMethod);

        AddRequestProperties();
        AddRequestBodyParameters(method.OriginalMethod);
        AddPathQueryAndHeaderParameters(generatorMethod);

        writer.Write(method);
        var result = tw.ToString();

        Assert.Contains("var command = new Command(\"user\");", result);
        Assert.Contains("command.Description = \"Test description\";", result);
        Assert.Contains("var qOption = new Option<string>(\"-q\", getDefaultValue: ()=> \"test\", description: \"The q option\")", result);
        Assert.Contains("qOption.IsRequired = false;", result);
        Assert.Contains("var jsonNoIndentOption = new Option<bool>(\"--json-no-indent\", r => {", result);
        Assert.Contains("command.AddOption(qOption);", result);
        Assert.Contains("command.AddOption(jsonNoIndentOption);", result);
        Assert.Contains("command.AddOption(outputOption);", result);
        Assert.Contains("command.SetHandler(async (object[] parameters) => {", result);
        Assert.Contains("var q = (string) parameters[0];", result);
        Assert.Contains("var requestInfo = CreateGetRequestInformation", result);
        Assert.Contains("requestInfo.PathParameters.Add(\"test%2Dpath\", testPath);", result);
        Assert.Contains("var response = await RequestAdapter.SendPrimitiveAsync<Stream>(requestInfo, errorMapping: default, cancellationToken: cancellationToken);", result);
        Assert.Contains("var formatterOptions = output.GetOutputFormatterOptions(new FormatterOptionsModel(!jsonNoIndent));", result);
        Assert.Contains("response = await outputFilter?.FilterOutputAsync(response, query, cancellationToken)", result);
        Assert.Contains("await formatter.WriteOutputAsync(response, formatterOptions, cancellationToken);", result);
        Assert.Contains("}, new CollectionBinding(qOption,", result);
        Assert.Contains("return command;", result);
    }

    [Fact]
    public void WritesExecutableCommandForPostRequestWithModelBody()
    {

        method.Kind = CodeMethodKind.CommandBuilder;
        method.SimpleName = "User";
        method.HttpMethod = HttpMethod.Post;
        var stringType = new CodeType
        {
            Name = "string",
        };
        var contentClass = root.AddClass(new CodeClass
        {
            Name = "Content",
            Kind = CodeClassKind.Model
        }).First();
        var bodyType = new CodeType
        {
            Name = "content",
            TypeDefinition = contentClass,
        };
        var generatorMethod = new CodeMethod
        {
            Kind = CodeMethodKind.RequestGenerator,
            Name = "CreatePostRequestInformation",
            HttpMethod = method.HttpMethod
        };
        method.OriginalMethod = new CodeMethod
        {
            Kind = CodeMethodKind.RequestExecutor,
            HttpMethod = method.HttpMethod,
            ReturnType = stringType,
            Parent = method.Parent
        };
        method.OriginalMethod.AddParameter(new CodeParameter
        {
            Name = "body",
            Kind = CodeParameterKind.RequestBody,
            Type = bodyType,
        });
        var codeClass = method.Parent as CodeClass;
        codeClass.AddMethod(generatorMethod);

        AddRequestProperties();
        AddPathQueryAndHeaderParameters(generatorMethod);

        writer.Write(method);
        var result = tw.ToString();

        Assert.Contains("var command = new Command(\"user\");", result);
        Assert.Contains("var qOption = new Option<string>(\"-q\", getDefaultValue: ()=> \"test\", description: \"The q option\")", result);
        Assert.Contains("qOption.IsRequired = false;", result);
        Assert.Contains("command.AddOption(qOption);", result);
        Assert.Contains("var bodyOption = new Option<string>(\"--body\")", result);
        Assert.Contains("bodyOption.IsRequired = true;", result);
        Assert.Contains("command.AddOption(bodyOption);", result);
        Assert.Contains("using var stream = new MemoryStream(Encoding.UTF8.GetBytes(body));", result);
        Assert.Contains("var model = parseNode.GetObjectValue<Content>(Content.CreateFromDiscriminatorValue);", result);
        Assert.Contains("var requestInfo = CreatePostRequestInformation", result);
        Assert.Contains("requestInfo.PathParameters.Add(\"test%2Dpath\", testPath);", result);
        Assert.Contains("var response = await RequestAdapter.SendPrimitiveAsync<Stream>(requestInfo, errorMapping: default, cancellationToken: cancellationToken);", result);
        Assert.Contains("return command;", result);
    }

    [Fact]
    public void WritesExecutableCommandForPostRequestWithCollectionModel()
    {

        method.Kind = CodeMethodKind.CommandBuilder;
        method.SimpleName = "User";
        method.HttpMethod = HttpMethod.Post;
        var stringType = new CodeType
        {
            Name = "string",
        };
        var contentClass = root.AddClass(new CodeClass
        {
            Name = "Content",
            Kind = CodeClassKind.Model
        }).First();
        var bodyType = new CodeType
        {
            Name = "content",
            TypeDefinition = contentClass,
            CollectionKind = CodeTypeBase.CodeTypeCollectionKind.Complex
        };
        var generatorMethod = new CodeMethod
        {
            Kind = CodeMethodKind.RequestGenerator,
            Name = "CreatePostRequestInformation",
            HttpMethod = method.HttpMethod
        };
        method.OriginalMethod = new CodeMethod
        {
            Kind = CodeMethodKind.RequestExecutor,
            HttpMethod = method.HttpMethod,
            ReturnType = stringType,
            Parent = method.Parent
        };
        method.OriginalMethod.AddParameter(new CodeParameter
        {
            Name = "body",
            Kind = CodeParameterKind.RequestBody,
            Type = bodyType,
        });
        var codeClass = method.Parent as CodeClass;
        codeClass.AddMethod(generatorMethod);

        AddRequestProperties();
        AddPathQueryAndHeaderParameters(generatorMethod);

        writer.Write(method);
        var result = tw.ToString();

        Assert.Contains("var command = new Command(\"user\");", result);
        Assert.Contains("var qOption = new Option<string>(\"-q\", getDefaultValue: ()=> \"test\", description: \"The q option\")", result);
        Assert.Contains("qOption.IsRequired = false;", result);
        Assert.Contains("command.AddOption(qOption);", result);
        Assert.Contains("var bodyOption = new Option<string>(\"--body\")", result);
        Assert.Contains("bodyOption.IsRequired = true;", result);
        Assert.Contains("command.AddOption(bodyOption);", result);
        Assert.Contains("using var stream = new MemoryStream(Encoding.UTF8.GetBytes(body));", result);
        Assert.Contains("var model = parseNode.GetCollectionOfObjectValues<Content>(Content.CreateFromDiscriminatorValue);", result);
        Assert.Contains("var requestInfo = CreatePostRequestInformation", result);
        Assert.Contains("requestInfo.PathParameters.Add(\"test%2Dpath\", testPath);", result);
        Assert.Contains("var response = await RequestAdapter.SendPrimitiveAsync<Stream>(requestInfo, errorMapping: default, cancellationToken: cancellationToken);", result);
        Assert.Contains("return command;", result);
    }

    [Fact]
    public void WritesExecutableCommandForPostRequestWithStreamBody()
    {

        method.Kind = CodeMethodKind.CommandBuilder;
        method.SimpleName = "User";
        method.HttpMethod = HttpMethod.Post;
        var stringType = new CodeType
        {
            Name = "string",
        };
        var bodyType = new CodeType
        {
            Name = "stream"
        };
        var generatorMethod = new CodeMethod
        {
            Kind = CodeMethodKind.RequestGenerator,
            Name = "CreatePostRequestInformation",
            HttpMethod = method.HttpMethod
        };
        method.OriginalMethod = new CodeMethod
        {
            Kind = CodeMethodKind.RequestExecutor,
            HttpMethod = method.HttpMethod,
            ReturnType = stringType,
            Parent = method.Parent
        };
        method.OriginalMethod.AddParameter(new CodeParameter
        {
            Name = "body",
            Kind = CodeParameterKind.RequestBody,
            Type = bodyType,
        });
        var codeClass = method.Parent as CodeClass;
        codeClass.AddMethod(generatorMethod);

        AddRequestProperties();
        AddPathQueryAndHeaderParameters(generatorMethod);

        writer.Write(method);
        var result = tw.ToString();

        Assert.Contains("var command = new Command(\"user\");", result);
        Assert.Contains("var qOption = new Option<string>(\"-q\", getDefaultValue: ()=> \"test\", description: \"The q option\")", result);
        Assert.Contains("qOption.IsRequired = false;", result);
        Assert.Contains("command.AddOption(qOption);", result);
        Assert.Contains("var bodyOption = new Option<Stream>(\"--file\")", result);
        Assert.Contains("bodyOption.IsRequired = true;", result);
        Assert.Contains("command.AddOption(bodyOption);", result);
        Assert.Contains("using var stream = file.OpenRead();", result);
        Assert.Contains("var requestInfo = CreatePostRequestInformation", result);
        Assert.Contains("requestInfo.PathParameters.Add(\"test%2Dpath\", testPath);", result);
        Assert.Contains("var response = await RequestAdapter.SendPrimitiveAsync<Stream>(requestInfo, errorMapping: default, cancellationToken: cancellationToken);", result);
        Assert.Contains("return command;", result);
    }

    [Fact]
    public void WritesExecutableCommandForDeleteRequest() {
        
        method.Kind = CodeMethodKind.CommandBuilder;
        method.SimpleName = "User";
        method.HttpMethod = HttpMethod.Delete;
        var stringType = new CodeType {
            Name = "void",
        };
        var generatorMethod = new CodeMethod {
            Kind = CodeMethodKind.RequestGenerator,
            Name = "CreateDeleteRequestInformation",
            HttpMethod = method.HttpMethod
        };
        method.OriginalMethod = new CodeMethod {
            Kind = CodeMethodKind.RequestExecutor,
            HttpMethod = method.HttpMethod,
            ReturnType = stringType,
            Parent = method.Parent
        };
        var codeClass = method.Parent as CodeClass;
        codeClass.AddMethod(generatorMethod);

        AddRequestProperties();
        AddPathQueryAndHeaderParameters(generatorMethod);

        writer.Write(method);
        var result = tw.ToString();

        Assert.Contains("var command = new Command(\"user\");", result);
        Assert.Contains("var qOption = new Option<string>(\"-q\", getDefaultValue: ()=> \"test\", description: \"The q option\")", result);
        Assert.Contains("qOption.IsRequired = false;", result);
        Assert.Contains("command.AddOption(qOption);", result);
        Assert.Contains("var requestInfo = CreateDeleteRequestInformation", result);
        Assert.Contains("requestInfo.PathParameters.Add(\"test%2Dpath\", testPath);", result);
        Assert.Contains("await RequestAdapter.SendNoContentAsync(requestInfo, errorMapping: default, cancellationToken: cancellationToken);", result);
        Assert.Contains("Console.WriteLine(\"Success\");", result);
        Assert.Contains("return command;", result);
        Assert.DoesNotContain("command.AddOption(outputOption);", result);
        Assert.DoesNotContain("var jsonNoIndentOption = new Option<bool>(\"--json-no-indent\", r => {", result);
    }

    [Fact]
    public void WritesExecutableCommandForGetStreamRequest()
    {

        method.Kind = CodeMethodKind.CommandBuilder;
        method.SimpleName = "User";
        method.HttpMethod = HttpMethod.Get;
        var streamType = new CodeType
        {
            Name = "stream",
        };
        var generatorMethod = new CodeMethod
        {
            Kind = CodeMethodKind.RequestGenerator,
            Name = "CreateGetRequestInformation",
            HttpMethod = method.HttpMethod
        };
        method.OriginalMethod = new CodeMethod
        {
            Kind = CodeMethodKind.RequestExecutor,
            HttpMethod = method.HttpMethod,
            ReturnType = streamType,
            Parent = method.Parent
        };
        var codeClass = method.Parent as CodeClass;
        codeClass.AddMethod(generatorMethod);

        AddRequestProperties();
        AddRequestBodyParameters(method.OriginalMethod);
        AddPathQueryAndHeaderParameters(generatorMethod);

        writer.Write(method);
        var result = tw.ToString();

        Assert.Contains("var command = new Command(\"user\");", result);
        Assert.Contains("var qOption = new Option<string>(\"-q\", getDefaultValue: ()=> \"test\", description: \"The q option\")", result);
        Assert.Contains("qOption.IsRequired = false;", result);
        Assert.Contains("command.AddOption(qOption);", result);
        Assert.Contains("var fileOption = new Option<FileInfo>(\"--file\");", result);
        Assert.Contains("command.AddOption(fileOption);", result);
        Assert.Contains("command.SetHandler(async (object[] parameters) => {", result);
        Assert.Contains("var q = (string) parameters[0];", result);
        Assert.Contains("var requestInfo = CreateGetRequestInformation", result);
        Assert.Contains("requestInfo.PathParameters.Add(\"test%2Dpath\", testPath);", result);
        Assert.Contains("var response = await RequestAdapter.SendPrimitiveAsync<Stream>(requestInfo, errorMapping: default, cancellationToken: cancellationToken);", result);
        Assert.Contains("}, new CollectionBinding(qOption,", result);
        Assert.Contains("return command;", result);
    }

    [Fact]
    public void WritesExecutableCommandForPostVoidRequest() {
        
        method.Kind = CodeMethodKind.CommandBuilder;
        method.SimpleName = "User";
        method.HttpMethod = HttpMethod.Post;
        var stringType = new CodeType {
            Name = "string",
        };
        var voidType = new CodeType {
            Name = "void",
        };
        var generatorMethod = new CodeMethod {
            Kind = CodeMethodKind.RequestGenerator,
            Name = "CreatePostRequestInformation",
            HttpMethod = method.HttpMethod
        };
        method.OriginalMethod = new CodeMethod {
            Kind = CodeMethodKind.RequestExecutor,
            HttpMethod = method.HttpMethod,
            ReturnType = voidType,
            Parent = method.Parent
        };
        method.OriginalMethod.AddParameter(new CodeParameter{
            Name = "body",
            Kind = CodeParameterKind.RequestBody,
            Type = stringType,
        });
        var codeClass = method.Parent as CodeClass;
        codeClass.AddMethod(generatorMethod);

        AddRequestProperties();
        AddPathQueryAndHeaderParameters(generatorMethod);

        writer.Write(method);
        var result = tw.ToString();

        Assert.Contains("var command = new Command(\"user\");", result);
        Assert.Contains("var qOption = new Option<string>(\"-q\", getDefaultValue: ()=> \"test\", description: \"The q option\")", result);
        Assert.Contains("qOption.IsRequired = false;", result);
        Assert.Contains("command.AddOption(qOption);", result);
        Assert.Contains("var bodyOption = new Option<string>(\"--body\")", result);
        Assert.Contains("bodyOption.IsRequired = true;", result);
        Assert.Contains("command.AddOption(bodyOption);", result);
        Assert.Contains("var requestInfo = CreatePostRequestInformation", result);
        Assert.Contains("requestInfo.PathParameters.Add(\"test%2Dpath\", testPath);", result);
        Assert.Contains("await RequestAdapter.SendNoContentAsync(requestInfo, errorMapping: default, cancellationToken: cancellationToken);", result);
        Assert.Contains("Console.WriteLine(\"Success\");", result);
        Assert.Contains("return command;", result);
        Assert.DoesNotContain("response = await outputFilter?.FilterOutputAsync(response, query, cancellationToken)", result);
        Assert.DoesNotContain("await formatter.WriteOutputAsync(response, formatterOptions, cancellationToken);", result);
    }
}
